/*#######################################################################################
AVR Ansteuerung paralleles Fischertechnik Interface 30520
Weitere Erkrungen siehe FTI.h

Copyright (C) 2005 Christian Truckenbrodt

#######################################################################################*/

#include "main.h"
#include "FTI.h"

//typedef unsigned char byte;

int motorbyte = 0x00;
int inputbyte = 0x00;

//############################################################################
//FTI Init Routine
void FTI_Init (void)
//############################################################################
{	
	//internen PullUP fr DataIn einschalten
	FTI_Port_Write = (1<<FTI_DataIn); 
	
	//Aus- bzw. Eingnge einstellen
	FTI_Port_DDR   = (1<<FTI_LoadOut)|(1<<FTI_LoadIn)|(1<<FTI_DataOut)|(1<<FTI_Clock)|(1<<FTI_TriggerX)|(1<<FTI_TriggerY);

	//init outputs
	//	FTI_TriggerX	= 1;
	//	FTI_TriggerY	= 1;
	FTI_Port_Write |= (1<<FTI_TriggerX);
	FTI_Port_Write |= (1<<FTI_TriggerY);
	//FTI_Clock		= 0;
	//FTI_LoadOut		= 0;
	//FTI_LoadIn		= 0;
	//FTI_DataOut		= 0;
	FTI_Port_Write |= (1<<FTI_Clock);
	FTI_Port_Write |= (1<<FTI_LoadOut);
	FTI_Port_Write &= ~(1<<FTI_LoadIn);
	FTI_Port_Write &= ~(1<<FTI_DataOut);
	//Alle Motoren Stop
	M(0,0,0);


}
//############################################################################
//FTI Wait Routine
void FTI_Wait (int value)
//############################################################################
{
	//x us warten, da sonst die Signale zu schnell fr das FTI sind
	for (int k=1;k<=(SYSCLK*value/1000000);k++){;;} 
}

//############################################################################
//FTI M Routine stellt den Befehl zur Steuerung einzelner
//Motoren bereit und erzeugt aus der Information das motorbyte
void M (int motor_no,int state,int direction)
//############################################################################
{
	//direction, state und motor_no in motorbyte hinzufgen
	if (motor_no > 0)
	{
		//die jeweiligen 2 bits an der motor_no STelle Null setzen
		motorbyte &= ~(0x03 << ((motor_no-1)*2));
		//Wenn der Motor nicht abgeschaltet werden soll, dann ...
		if (state != 0)
		{
			//... mache direction ungleich state
			if (direction == 1) state = 0;
			//und fge die neuen 2 Bits dem motorbyte hinzu
			motorbyte |= (state + (direction<<1)) << ((motor_no-1)*2);
		}
	}
	else
	{
		//Alle Motoren Stop
		motorbyte = 0x00;
	}
	//neues motorbyte an FTI bertragen
	FTI_TXRX ();
}

//############################################################################
//FTI EA Routine stellt den Befehl zum Auslesen der beiden Analogeingnge bereit.
// EA(0) gibt den Wert des Analogeingangs EX zurck EA(1) von EY.
int EA (int input)
//############################################################################
{
   	FTI_Port_Write |= (1<<FTI_TriggerX);
	FTI_Port_Write |= (1<<FTI_TriggerY);

   
	if (input == 0)
	{
		
		FTI_Port_Write &= ~(1<<FTI_TriggerX);
		FTI_Port_Write |= (1<<FTI_TriggerY);

	}
	else
	{
		FTI_Port_Write |= (1<<FTI_TriggerX);
		FTI_Port_Write &= ~(1<<FTI_TriggerY);

	}

	FTI_Port_Write &= ~(1<<FTI_Clock);
	FTI_Wait (1);
	
	FTI_Port_Write |= (1<<FTI_Clock);
	FTI_Port_Write |= (1<<FTI_TriggerX);
	FTI_Port_Write |= (1<<FTI_TriggerY);
	
	
	
	input = 0;
	
	while ( ((FTI_Port_Read & (1<<FTI_DataIn)) == 0))
	{
		input++;
	}
	return input;
}

//############################################################################
//FTI E Routine stellt den Befehl zum Auslesen einzelner
//Schalter bereit und erzeugt aus der Information das inputbyte
//int input ist die Schalternummer(1-8) die gelesen werden soll.
int E (int input)
//############################################################################
{
   //inputbyte einlesen
   FTI_TXRX ();
   // Wenn das inputbyte am Bit "input" > 0 dann gib 1 zurck, ansonsten 0
   if ( (inputbyte & (0x01<<(input-1))) > 0)
   {
     input = 1;
   }
   else
   {
     input = 0;
   }
   
   //Rckgabewert
   return input;
}

//############################################################################
//FTI_TXRX liet das inputbyte, bertrgt das motorbyte und setzt den 
//Watchdog im Interface zurck. Die Routine muss mindestens alle 0,3s 
//aufgerufen werden.
void FTI_TXRX (void)
//############################################################################
{
	int bit;
	
	//init outputs
	FTI_Port_Write |= (1<<FTI_TriggerX);
	FTI_Port_Write |= (1<<FTI_TriggerY);

	
	//bit 15 bis 0 senden
	for (bit=15;bit>=0;bit--)
    {
		
		//Wenn bitx high, dann dataout = 1 else =0
		if( (motorbyte & (0x01<<bit)) > 0)
		{
			//FTI_DataOut = 1;
			FTI_Port_Write &= ~(1<<FTI_LoadOut);
			FTI_Port_Write &= ~(1<<FTI_Clock);
			FTI_Port_Write |= (1<<FTI_DataOut);
			FTI_Wait (1);
			//FTI_Clock =1;
			FTI_Port_Write |= (1<<FTI_Clock);
			FTI_Wait (1);
			
		}
		else
		{
			//FTI_DataOut = 0;
			FTI_Port_Write &= ~(1<<FTI_DataOut);
			FTI_Port_Write &= ~(1<<FTI_LoadOut);
			FTI_Port_Write &= ~(1<<FTI_Clock);
			FTI_Wait (1);
			FTI_Port_Write |= (1<<FTI_Clock);
			FTI_Wait (1);
			
		}
	}
    
	//FTI_DataOut = 0;
	FTI_Port_Write &= ~(1<<FTI_DataOut);
	FTI_Port_Write |= (1<<FTI_LoadOut);
	FTI_Port_Write |= (1<<FTI_Clock);
	FTI_Wait (1);


	//Eingnge abfragen
	//Alle Eingnge als 0 annehmen
	inputbyte = 0x0000;

	FTI_Port_Write &= ~(1<<FTI_DataOut);
	FTI_Port_Write &= ~(1<<FTI_LoadOut);
	FTI_Port_Write |= (1<<FTI_LoadIn);
	FTI_Port_Write &= ~(1<<FTI_Clock);
	FTI_Wait (1);
	FTI_Port_Write |= (1<<FTI_Clock);
	FTI_Wait (1);
	
	//inputbyte bitweise empfangen
	for (bit=7;bit>=0;bit--)
	{
		//Gucken welcher Eingang nicht 0 ist und korrigieren
		if ( (FTI_Port_Read & (1<<FTI_DataIn)) == 0)
		{
			inputbyte = inputbyte | (0x01 << bit);
		}	
		FTI_Port_Write &= ~(1<<FTI_DataOut);
		FTI_Port_Write &= ~(1<<FTI_LoadOut);
		FTI_Port_Write &= ~(1<<FTI_LoadIn);
		FTI_Port_Write &= ~(1<<FTI_Clock);
		FTI_Wait (1);
		FTI_Port_Write |= (1<<FTI_Clock);
		FTI_Wait (1);
	}
	
	//inputbyte bitweise empfangen
	for (bit=15;bit>=8;bit--)
	{
		//Gucken welcher Eingang nicht 0 ist und korrigieren
		if ( (FTI_Port_Read & (1<<FTI_DataIn)) == 0)
		{
			inputbyte = inputbyte | (0x01 << bit);
		}
		FTI_Port_Write &= ~(1<<FTI_DataOut);
		FTI_Port_Write &= ~(1<<FTI_LoadOut);
		FTI_Port_Write &= ~(1<<FTI_LoadIn);
		FTI_Port_Write &= ~(1<<FTI_Clock);
		FTI_Wait (1);
		FTI_Port_Write |= (1<<FTI_Clock);
		FTI_Wait (1);
	}
}
