/*#######################################################################################
 AVR Ansteuerung paralleles Fischertechnik Interface 30520
 
 Copyright:      Christian Truckenbrodt  mailto: trucki@web.de
				 Diese Routine darf nicht ohne mein Einverstndnis fr kommerzielle
				 Zwecke verwendet werden.

 Version:        14.04.2005
 
 System:		 WinAVR, Ponyprog2000, ISP, ATMEGA32@16MHZ, 2x16LCD an PORTC,
				 LCD Routine von Radig Ulrich.
				 
 Description:  	 Programm zur Ansteuerung des parallelen Fischertechnik Interface 30520.
				 ATMEGA wird mittels 7 normalen IO PORTS an das FTI angeschlossen.
				 Die Pins werden in FTI.h festgelegt.
				 
				 Das parallele FTI wird wie folgt mit dem ATMEGA32 verbunden:
				 
				 ---         									    ---
				 A |PA0---------LoadOut(D-Sub25 Pin  2)-(20P Pin 13)| F
				 T |PA1----------LoadIn(D-Sub25 Pin  3)-(20P Pin 14)| T
				 M |PA2---------DataOut(D-Sub25 Pin  4)-(20P Pin 11)| I
				 E |PA3-----------Clock(D-Sub25 Pin  5)-(20P Pin 12)| 3
				 G |PA4--------TriggerX(D-Sub25 Pin  6)-(20P Pin  9)| 0  
				 A |PA5--------TriggerY(D-Sub25 Pin  7)-(20P Pin 10)| 5   
				   |PA64---------DataIn(D-Sub25 Pin 11)-(20P Pin  3)| 2
				   |				         			 	        | 0
				 3 |GND--------------------------------+8V-UBextern | +    
				 2 |   |---GND(D-Sub25 Pin 17-25)(20P Pin 1,2,19,20)| -    
				 ---   -   					    				    ---  
				 
				Bevor das FTI genutzt werden kann, muss FTI_Init() aufgerufen werden.
				Anschlieend knnen Motoren mit M(Motor_Nr,An/Aus, Richtung) angesteuert 
				werden. Motor_Nr = 0 schaltet alle Motoren aus.
				
				Mit E(schalter_Nr) kann der Status eines Schalters ausgelesen werden.
				Mit EA(1) wird der analoger Eingang EY gelesen, mit EA(0) EX. Der Wert 
				liegt je nach Prozessorgeschwindigkeit zwischen 0 und 20000.
				
				Die Funktion FTI_TXRX() muss mindestens alle 300ms aufgerufen werden, da 
				sonst das FTI davon ausgeht, dass der Steuercomputer abgestrzt ist. 
				 
				Ich hafte fr keinerlei Schden! Die Benutzung dieser Routine geschieht 
				auf eigene Gefahr!
				 
Copyright (C) 2005 Christian Truckenbrodt			 
#######################################################################################*/
#ifndef _FTI_H
 #define _FTI_H

extern void FTI_Init (void);
extern void FTI_TXRX (void);
extern void M (int,int,int);  // int motor_no,state,direction
extern int E (int);
extern int EA (int);


#define FTI_Port_DDR			DDRB	//Port an dem das parallele Fischertechnik Interface 30520 angeschlossen wird
#define FTI_Port_Write			PORTB
#define FTI_Port_Read			PINB

#define FTI_LoadOut					0		//Pin fuer LoadOut
#define FTI_LoadIn 					1		//Pin
#define FTI_DataOut 				2		//Pin
#define FTI_Clock 					3		//Pin
#define FTI_TriggerX 				4		//Pin
#define FTI_TriggerY 				5		//Pin

#define FTI_DataIn 					6		//Pin

#endif //_FTI_H






